; main.asm
;
; Main programs for Smart Boot Manager
;
; Copyright (C) 2000, Suzhe. See file COPYING for details.
;
%define MAIN
%define HAVE_MAIN_PROG
;%define EMULATE_PROG

%include "ui.h"
%include "hd_io.h"
%include "knl.h"
%include "sbm.h"

%define MAIN_WIN_WIDTH      49
%define MAIN_WIN_HEIGHT     12
%define MENU_POS_ROW        3
%define MENU_POS_COL        2
%define MENU_TITLE_POS_ROW  2
%define MENU_TITLE_POS_COL  2
%define MENU_WIDTH          45
%define MENU_HEIGHT         7
%define DELAY_TIME_ROW      11
%define DELAY_TIME_COL      37
%define ROOT_PASSWD_ROW     11
%define ROOT_PASSWD_COL     2

%define MAX_RECORD_NUM      32
%define MAX_PASSWORD_LENGTH 16

%define NUM_OF_HOTKEYS      (func_list.entries - func_list.hotkeys)/2

%define KNLFLAG_FIRSTRUN    0x01
%define KNLFLAG_FLOPPY_OK   0x04

        bits 16

%ifdef EMULATE_PROG
        org 0x100
%else
        org 0
%endif

        section .text
        
start_of_kernel:
start_of_checksum_area:

        jmp start
        nop
;!!! PLEASE DON NOT CHANGE THE SIZE AND ORDER OF FOLLOWING DATA !!!

;=============================================================================
;the header of Smart Boot Manager kernel ( 8 bytes )
;=============================================================================
sbmk_magic          dd  SBMK_MAGIC ; magic number = 'SBMK', 4 bytes.
                                   ; it's abbr. of 'Smart Boot Manager Kernel'
sbmk_version        dw  SBMK_VERSION ; version, high byte is major version,
                                   ; low byte is minor version.
kernel_size         dw  (end_of_kernel-start_of_kernel)
checksum_size       dw  (end_of_checksum_area - start_of_checksum_area)
checksum            db  0          ; checksum of the kernel program.

kernel_sectors      db  0          ; the number of sectors that kernel and theme
                                   ; totally used.
kernel_copies       db  0          ; number of kernel copies saved. <= 3
kernel_drvid        db  0          ; the drive id that kernel saved.
kernel_addr         dd  0          ; the abs sector addr that kernel saved.

kernel_drvid2       db  0          ;
kernel_addr2        dd  0          ;

kernel_drvid3       db  0          ;
kernel_addr3        dd  0          ;

;=============================================================================
; MAIN data area
;=============================================================================
kernel_flags        db 0x07                     ; first run signature and video
                                                ; mode and floppy ok signature.
delay_time          db 30                       ; delay time ( seconds )
direct_boot_record  db 0xff                     ; >= MAX_RECORD_NUM means no
                                                ; direct boot.
default_record      db 0xff                     ; the record number will
                                                ; be booted after the
                                                ; delay time is up or ESC
                                                ; key is pressed.
root_password       dd 0

; buffer to store boot records
boot_records        times MAX_RECORD_NUM * SIZE_OF_BOOTRECORD db 0
;=============================================================================

start:

%ifndef EMULATE_PROG                            ; real program goes here.

        cli
        mov ax, cs                              ; initialize the segment
        mov ds, ax                              ; registers and the stack
        mov es, ax                              ; pointer.
        xor ax, ax
        mov ss, ax                              ; ss:sp = 0:0xA000
        mov sp, KERNEL_OFF                      ;
        sti
        
        test byte [kernel_flags], KNLFLAG_FIRSTRUN
        jz .not_first_run
        
        call init_theme                         ; initialize the theme, only time!
        call init_video                         ; initialize the video mode.
        call init_boot_records                  ; if it's the first time
        and byte [kernel_flags], ~ KNLFLAG_FIRSTRUN ; to run this program,
        call save_boot_manager                  ; call the init_boot_records.
        jc .disk_error
        jmp .show_menu
        
.not_first_run:
        
        call init_video                         ; initialize the video mode.

        mov ah, 0x02                            ; test the keyboard status,
        int 0x16                                ; if ctrl pressed then show
        test al, kbCtrlMask                     ; menu directly,
        jnz .show_menu                          ;
        
        mov bl, SIZE_OF_BOOTRECORD

        mov al, [direct_boot_record]            ; check if need boot directly.
        cmp al, MAX_RECORD_NUM                  ;
        jb .go_direct_boot

        cmp byte [delay_time], 0
        jnz .show_menu                          ; delay_time = 0, boot the
                                                ; default record directly.
        mov al, [default_record]                ;
        cmp al, MAX_RECORD_NUM
        jb .go_def_boot
        jmp .show_menu
        
.go_direct_boot:
        mov byte [direct_boot_record], 0xff     ; clear the direct boot sig.
        call save_boot_manager
        jc .disk_error

.go_def_boot:
        mul bl                                  ; get the pointer to the
        lea si, [boot_records]                  ; direct/default boot record.
        add si, ax                              ;

        call check_bootrecord                   ; check if it's valid
        jc .invalid_record

        call boot_the_record                    ; boot it.
        
        or al, al
        jnc .disk_error
        mov si, [str_idx.no_system]
        jmp .report_error
        
.disk_error:
        mov si, [str_idx.disk_error]
        jmp .report_error
.invalid_record:
        mov si, [str_idx.invalid_record]
.report_error:
        call error_box

%else
        call init_video                         ; here is the code for
        call init_boot_records                  ; emulate program.
%endif

.show_menu:
        mov byte [change_occured], 0
        
        mov al, [delay_time]                    ;
        cmp al, 255                             ;
        jae .not_count_time                     ;
                                                ; if delay_time = 255
        mov [time_count], al                    ; then do not count time.
        xor al, al                              ;
.not_count_time:                                ;
        mov [key_pressed], al                   ;
        
        xor al, al
        mov [first_visible_rec], al
        mov [focus_record], al
        
        call init_good_record_list

        mov bl, [default_record]
        lea si, [good_record_list]
        mov cl, [good_record_number]
        xor ch, ch
        xor bh, bh
        
.loop_search_def:
        lodsb
        cmp al, bl
        je .found_def
        inc bh
        loop .loop_search_def
        jmp .go_ahead

.found_def:
        mov [focus_record], bh
        cmp bh, [size.menu_height]
        jb .go_ahead
        inc bh
        sub bh, [size.menu_height]
        mov [first_visible_rec], bh

.go_ahead:
        call draw_screen
        call msg_dispatcher                     ; go to message dispatcher
                                                ; this function may not return.

;=============================================================================
;init_theme ---- initialize the theme data.
;=============================================================================
init_theme:
        mov bx, [icon.brand]
        or bx, bx
        jz .adjust_bkgnd                            ; no brand icon
        add word [icon.brand], theme_start          ;
.adjust_bkgnd:
        mov bx, [icon.background]
        or bx, bx
        jz .adjust_font                             ; no background icon
        add word [icon.background], theme_start
.adjust_font:
        mov bx, [font.data]
        or bx, bx
        jz .adjust_str
        add word [font.data], theme_start           ; no font data

.adjust_str:
        lea si, [str_idx]
        mov cx, (end_of_str_idx - str_idx)/2
        
.loop_adjust:
        mov bx, [si]
        add bx, theme_start
        mov [si], bx
        add si, 2
        loop .loop_adjust
        ret
        
;=============================================================================
;init_video ---- init the video mode
;input:
;      al = video mode ( 0: 90x25, otherwise: 80x25 )
;output:
;      none
;=============================================================================
init_video:
        mov al, [video_mode]
        mov bl, 8
        mov bp, [font.data]
        mov cx, [font.number]
        
        call set_video_mode
        ret

;=============================================================================
;init_good_record_list ---- init the good boot record list
;input:
;      none
;output:
;      cf = 0 sucess
;      cf = 1 failed, no good record
;=============================================================================
init_good_record_list:
        pusha
        mov cx, MAX_RECORD_NUM
        lea di, [good_record_list]
        lea si, [boot_records]
        xor ax, ax

.loop_check:
        call check_bootrecord                   ; check if it's valid
        jc .check_next

        stosb                                   ; store it's index to buffer
        inc ah
        
.check_next:
        inc al
        add si, SIZE_OF_BOOTRECORD
        loop .loop_check

        mov [good_record_number], ah
        or ah, ah
        jnz .ok
        stc
.ok:
        popa
        ret

;=============================================================================
;init_boot_records ---- init the boot records list
;input:
;      none
;output:
;      cf = 0 success
;      cf = 1 failed, no good record
;=============================================================================
init_boot_records:
        pusha
        inc byte [change_occured]               ; some changes occured.
        
        mov al, SIZE_OF_BOOTRECORD
        mov cl, MAX_RECORD_NUM
        mul cl
        push cx
        mov cx, ax
        lea di, [boot_records]
        push di
        xor al, al
        rep stosb
        pop di
        pop cx
        
        xor dl, dl
        test byte [kernel_flags], KNLFLAG_FLOPPY_OK
        jnz .search_it
        mov dl, MIN_HD_ID
.search_it:
        call search_drives
        or ch, ch
        jnz .ok
        stc
.ok:
        popa
        ret

;=============================================================================
;draw_screen ---- draw the whole screen
;input:
;      none
;output:
;      none
;=============================================================================
draw_screen:
        mov bh, [color.background]              ;
        mov si, [icon.background]               ; draw background
        mov cx, [icon.background_size]          ;
        call draw_background                    ;

        xor dx, dx                              ;
        mov bl, [color.copyright]               ;
        mov al, [screen_width]                  ; draw copyright message
        push ax                                 ; save screen width
        mov cl, [size.copyright]
        mul cl
        mov cx, ax
        mov al, ' '                             ;
        call draw_char                          ;
        mov si, [str_idx.copyright]
        call draw_string
        
        mov bl, [color.hint]                    ;
        mov dh, [screen_height]                 ;
        mov cl, [size.hint]                     ;
        sub dh, cl                              ; draw hint message
        pop ax                                  ; get screen width
        mul cl                                  ;
        mov cx, ax                              ;
        mov al, ' '                             ;
        call draw_char                          ;
        mov si, [str_idx.hint]                  ;
        call draw_string                        ;

        mov dx, [position.brand_col]            ; draw brand icon
        mov cx, [icon.brand_size]               ;
        cmp dl, 0xFF                            ;
        jne .not_justify                        ;
        mov dl, [screen_width]                  ; right justify
        sub dl, cl                              ;
.not_justify:                                   ;
        mov si, [icon.brand]                    ;
        call draw_icon                          ;

        call draw_delay_time
        call draw_main_window
        call draw_menu
        ret
        
;=============================================================================
;draw_main_window ---- draw the main window
;input:
;      none
;output:
;      none
;=============================================================================
draw_main_window:
        pusha
        mov cx, [position.main_win_col]                 ; get main window's
        mov dx, cx                                      ; coordinate
        add dh, [size.main_win_height]
        dec dh
        add dl, (MAIN_WIN_WIDTH-1)

        mov bx, [color.main_win_frame]                  ; get main window's color
        mov si, [str_idx.main_win_title]                ; get main window's title

        call draw_window

        cmp dword [root_password], 0
        jz .no_root_password
        
        push cx
        mov dx, cx
        add dx, (ROOT_PASSWD_ROW << 8) + ROOT_PASSWD_COL
        mov cx, 1
        mov al, '!'
        call draw_char
        pop cx
        
.no_root_password:
        add cx, (MENU_TITLE_POS_ROW << 8) + MENU_TITLE_POS_COL
        mov dx, cx
        mov bl, [color.menu_title]
        mov al, ' '
        mov cx, MENU_WIDTH
        call draw_char
        mov si, [str_idx.menu_title]
        call draw_string
        popa
        ret

;=============================================================================
;draw_menu_item ---- draw a boot menu item
;input:
;      bl = high 4 bit Background color and low 4 bit Foreground color
;      cl = item index in good_record_list
;output:
;      none
;=============================================================================
draw_menu_item:
        pusha
        cmp byte [good_record_number], 0
        je .no_menu_item

        xor ch, ch
        lea si, [good_record_list]                  ; get real index of the
        add si, cx                                  ; boot record.
        lodsb                                       ; al = the index
        
        xor ah, ah                                  ; get the pointer of
        push ax                                     ;
        mov dl, SIZE_OF_BOOTRECORD                  ; selected boot record.
        mul dl                                      ;
        lea si, [boot_records]                      ;
        add si, ax                                  ;
        
        lea di, [tmp_buffer]
        call get_record_string

        mov dh, cl                                  ; calculate the draw
        sub dh, [first_visible_rec]                 ; coordinate of the
        add dh, MENU_POS_ROW                        ; menu item.
        mov dl, MENU_POS_COL                        ;
        add dx, [position.main_win_col]             ;

        mov cl, 1
        mov al, ' '
        call draw_char                              ; draw a leading space
        inc dl
        pop ax                                      ; al = the real index

        cmp al, [default_record]                    ;
        jne .not_default                            ;
        mov al, '*'                                 ; draw a flag before
        jmp .draw_def_flag                          ; the menu item, if
.not_default:                                       ; it's default record.
        mov al, ' '                                 ;
.draw_def_flag:                                     ;
        call draw_char                              ;
        inc dl                                      ;
        
        mov si, di
        call draw_string                            ; draw the menu item.

        call strlen                                 ; get the length of the item.
        mov al, ' '
        mov ah, MENU_WIDTH
        sub ah, cl
        sub ah, 3
        add dl, cl
        mov cl, ah
        inc cl
        call draw_char                              ; fill ending space chars
.no_menu_item:
        popa
        ret

;=============================================================================
;draw_menu ---- draw the boot menu
;input:
;      none
;output:
;      none
;=============================================================================
draw_menu:
        pusha
        mov bl, [color.menu_normal]
        mov bh, bl
        mov cx, [position.main_win_col]                 ; get the top left
        add cx, (MENU_POS_ROW << 8) + MENU_POS_COL      ; and bottom right
        mov dx, cx                                      ; corner's coordinate
        add dh, [size.menu_height]
        dec dh
        add dl, (MENU_WIDTH-1)                          ; of menu area

        mov ax,0x0600                                   ; clear the menu area
        int 0x10                                        ;

        mov cl, [first_visible_rec]
        mov ch, cl
        add ch, [size.menu_height]
.loop_draw:
        cmp cl, [good_record_number]
        jae .end
        cmp cl, ch
        jae .end
        call draw_menu_item
        inc cl
        jmp .loop_draw
.end:
        popa
        ret

;=============================================================================
;get_key ---- get a key stroke
;input:
;      none
;output:
;      ax = the key code
;=============================================================================
get_key:

.wait_key:
        cmp byte [delay_time], 0                        ; if delay_time set to zero
        je .no_count                                    ; then don't count time.
        cmp byte [key_pressed], 0                       ; if key pressed then don't
        jne .no_count                                   ; count time.

        xor ah, ah                                      ; get time ticks
        int 0x1a                                        ;

        cmp dx, [ticks_count]                           ;
        jae .next_time                                  ; dx must greater than
        mov [ticks_count], dx                           ; ticks_count
.next_time:
        mov cx, dx                                      ; every 18 ticks approxmiately
        sub cx, [ticks_count]                           ; equal to 1 second,
        cmp cx, 18                                      ; decrease time_count
        jbe .not_add                                    ; until to zero.
        mov [ticks_count], dx                           ;
        dec byte [time_count]                           ;
        call draw_delay_time                            ;
.not_add:
        cmp byte [time_count], 0                        ; if time is up, then
        jbe .esc_key                                    ; send ESC key.

.no_count:
        mov ah,1                                        ; if no key pressed
        int 0x16                                        ; go back to check
        jz .wait_key                                    ; time.
        xor ah,ah                                       ; if any key pressed
        int 0x16                                        ; then get it.
        mov byte [key_pressed], 1
        ret
.esc_key:
        mov ax, kbEsc
        inc byte [key_pressed]
        ret

;=============================================================================
;draw_delay_time ---- draw the delay_time and time_count
;=============================================================================
draw_delay_time:
        mov al, [time_count]
        mov cx, 3
        xor ah, ah
        lea di, [tmp_buffer]
        call itoa
        
        mov bl, [color.delay_time]
        mov dh, [screen_height]
        mov dl, [screen_width]
        sub dx, 0x0108
        mov si, di
        call draw_string

        mov al, [delay_time]
        mov cx, 3
        xor ah, ah
        call itoa

        mov al, '|'
        mov cl, 1
        add dl, 3
        call draw_char
        inc dl
        call draw_string
        mov al, ' '
        add dl, 3
        call draw_char
        ret
        
        
;=============================================================================
;msg_dispatcher ---- the main loop to handle user command
;input:
;      none
;output:
;      none
;=============================================================================
msg_dispatcher:
.loop_dispatch:
        mov bl, [color.menu_focus]
        mov cl, [focus_record]

        call draw_menu_item                     ; draw focus menu item
        push cx
        call get_key                            ; get a key
        pop cx
        mov bl, [color.menu_normal]
        call  draw_menu_item                    ; clear focus menu item

        lea si, [func_list.hotkeys]             ; search in hotkey list
        mov dx, ax                              ;
        mov cx, NUM_OF_HOTKEYS                  ;
        xor bx, bx                              ;
.loop_search:                                   ;
        lodsw                                   ;
        cmp dx, ax                              ;
        je .found_it                            ; found, go get func entry
        inc  bl                                 ;
        loop .loop_search                       ;
        jmp .loop_dispatch                      ; not found, wait more
.found_it:
        shl bl, 1
        call word [bx + func_list.entries]      ; call the function
        jmp .loop_dispatch


;=============================================================================
;below are key stroke handlers
;=============================================================================

;=============================================================================
;show_help ---- show the help window
;=============================================================================
show_help:
        mov si, [str_idx.help]
        mov di, [str_idx.help_content]
        or di, di
        jz .end
        
        mov al, [color.help_msg]
        mov bx, [color.help_win_frame]
        mov dx, [size.help_win]
        call message_box
        xor ah, ah
        int 0x16
        call draw_screen
.end:
        ret

;=============================================================================
;show_about ---- show the about window
;=============================================================================
show_about:
        mov si, [str_idx.about]
        mov di, [str_idx.about_content]
        or di, di
        jz .end
        
        mov al, [color.about_msg]
        mov bx, [color.about_win_frame]
        mov dx, [size.about_win]
        call message_box
        xor ah, ah
        int 0x16
        call draw_screen
.end:
        ret

;=============================================================================
;calc_checksum ---- calculate the checksum of the kernel program.
;input:
;      none
;output:
;      bl = the checksum value.
;=============================================================================
calc_checksum:
        lea si, [start_of_checksum_area]
        mov cx, [checksum_size]
        xor bl, bl
        mov [checksum], bl

.loop_calc:
        lodsb
        add bl, al
        loop .loop_calc
        
        ret
        
;=============================================================================
;save_boot_manager ---- save boot manager to disk.
;input:
;      none
;output:
;      cf = 0 success
;      cf = 1 failed
;=============================================================================
save_boot_manager:
        pusha
        call calc_checksum                      ; calculate the checksum.
        neg bl
        mov [checksum], bl                      ;

        lea si, [kernel_drvid]
        xor ch, ch
        mov cl, [kernel_copies]

.loop_save:
        push cx
        
        lodsb
        mov dl, al
        lodsw
        mov bx, ax
        lodsw
        mov cx, ax
        
        mov dh, [kernel_sectors]
        mov ah, INT13H_WRITE
        mov al, 1
        xor di, di

.loop_write:
        clc
        or dh, dh
        jz .write_end
        call disk_access

        jc .write_end
        
        add di, SECTOR_SIZE
        adc bx, 1                      ; inc cx : bx, read next sector.
        adc cx, 0                      ;
        dec dh
        jmp .loop_write
        
.write_end:

        pop cx
        
        jc .disk_error
        loop .loop_save

        mov byte [change_occured], 0               ; clear change signature.
        
        clc
.disk_error:
        popa
        ret
        
;=============================================================================
;ask_save_changes ---- save boot manager to disk
;=============================================================================
ask_save_changes:
        cmp byte [change_occured], 0
        je .no_changes
        
        mov si, [str_idx.ask_save_changes]
        call info_box
        cmp ax, kbEnter
        je save_changes
        cmp ax, kbLowerY
        je save_changes
        cmp ax, kbUpperY
        je save_changes
        
.no_changes:
        ret

;=============================================================================
;save_changes ---- save boot manager to disk
;=============================================================================
save_changes:
;        call confirm_root_passwd
;        jc .end_clean

%ifndef EMULATE_PROG
        call save_boot_manager
        jc .disk_error
%endif

        mov si, [str_idx.changes_saved]
        call info_box
        ret
        
.disk_error:
        mov si, [str_idx.disk_error]
        call error_box
.end:
        ret

;=============================================================================
;change_video_mode ---- change the video mode
;=============================================================================
change_video_mode:

        inc byte [change_occured]               ; some changes occured.

        mov al, [video_mode]
        not al
        mov [video_mode], al
        call init_video
        call draw_screen
        ret
        
;=============================================================================
;up_cursor ---- move the cursor up
;=============================================================================
up_cursor:
        mov al, [focus_record]
        or al, al                               ; already at top of the list
        jz .end                                 ; do nothing
        dec al
        mov [focus_record], al
        cmp [first_visible_rec], al
        jbe .end                                ; no need to scroll down.
        mov [first_visible_rec], al             ;
        call draw_menu                          ; scroll down.
.end:
        ret
        
;=============================================================================
;down_cursor ---- move the cursor down
;=============================================================================
down_cursor:
        mov al, [focus_record]
        inc al
        cmp al, [good_record_number]            ;
        jae .end                                ; already at bottom of the list
        mov [focus_record], al
        sub al, [first_visible_rec]
        cmp al, [size.menu_height]
        jb .end                                 ; no need to scroll up
        inc byte [first_visible_rec]
        call draw_menu
.end:
        ret

;=============================================================================
;change_name ---- change the record name
;=============================================================================
change_name:
        cmp byte [good_record_number], 0
        je .end
        
        call confirm_record_passwd
        jc .end

        call get_record_pointer

        add si, struc_bootrecord.name                   ;
        lea di, [tmp_buffer]                            ; get old name.
        mov cl, MAX_NAME_LENGTH                         ;
        xor ch, ch                                      ;
        push cx                                         ;
        push di                                         ;
        call strncpy                                    ;
        pop di                                          ;
        pop cx                                          ;
        
        push si
        
        xor ah, ah
        mov al, [color.input_box_msg]
        mov bx, [color.input_box_frame]
        mov si, [str_idx.name]
        call input_box
        pop si
        jc .end_clean


        xchg di, si
        xor ch, ch
        call strncpy

        inc byte [change_occured]               ; some changes occured.
        
.end_clean:
        call draw_screen
.end:
        ret

;=============================================================================
;change_root_passwd ---- change the root password
;=============================================================================
change_root_passwd:
        call confirm_root_passwd
        jc .end
        
        call draw_screen
        
        mov si, [str_idx.new_root_passwd]
        call input_passwd
        jc .end
        mov bx, ax
        mov cx, dx

        push bx
        push cx
        mov si, [str_idx.retype_passwd]
        call input_passwd
        pop cx
        pop bx
        jc .end
        cmp bx, ax
        jne .wrong
        cmp cx, dx
        jne .wrong
        mov [root_password], bx
        mov [root_password+2], cx
        mov si, [str_idx.passwd_changed]
        call info_box

        inc byte [change_occured]               ; some changes occured.
        
        jmp .end
.wrong:
        mov si, [str_idx.wrong_passwd]
        call error_box
.end:
        ret

;=============================================================================
;change_record_passwd ---- change the record password
;=============================================================================
change_record_passwd:
        cmp byte [good_record_number], 0
        je .end

        call  confirm_record_passwd
        jc .end

        call draw_screen

        mov si, [str_idx.new_record_passwd]
        call input_passwd
        jc .end
        mov bx, ax
        mov cx, dx

        push bx
        push cx
        mov si, [str_idx.retype_passwd]
        call input_passwd
        pop cx
        pop bx
        jc .end
        cmp bx, ax
        jne .wrong
        cmp cx, dx
        jne .wrong

        call get_record_pointer
        mov [si+struc_bootrecord.password], bx
        mov [si+struc_bootrecord.password+2], cx
        
        mov si, [str_idx.passwd_changed]
        call info_box

        inc byte [change_occured]               ; some changes occured.
        
        ret
        
.wrong:
        mov si, [str_idx.wrong_passwd]
        call error_box
.end:
        ret

;=============================================================================
;set_default_record ---- set the default boot record
;=============================================================================
set_default_record:
        cmp byte [good_record_number], 0
        je .end

        call confirm_root_passwd
        jc .end

        mov dl, [focus_record]                  ;
        xor dh, dh                              ; get real record index
        lea si, [good_record_list]              ;
        add si, dx                              ;
        lodsb                                   ;
        
        mov [default_record], al
        inc byte [change_occured]               ; some changes occured.
        call draw_menu
.end:
        ret

;=============================================================================
;unset_default_record ---- unset the default boot record
;=============================================================================
unset_default_record:
        call confirm_root_passwd
        jc .end

        mov byte [default_record], 0xFF
        inc byte [change_occured]               ; some changes occured.
        call draw_menu
.end:
        ret
        
;=============================================================================
;toggle_auto_active ---- toggle the auto active switch
;=============================================================================
toggle_auto_active:
        cmp byte [good_record_number], 0
        je .end

        call get_record_pointer
        call check_allow_act
        jc .end

        push si
        call confirm_record_passwd
        pop si
        jc .end
        
        xor byte [si + struc_bootrecord.flags], INFOFLAG_AUTOACTIVE
        inc byte [change_occured]               ; some changes occured.
        
.end:
        ret

;=============================================================================
;toggle_auto_hide ---- toggle the auto hide switch
;=============================================================================
toggle_auto_hide:
        cmp byte [good_record_number], 0
        je .end

        call get_record_pointer
        call check_allow_hide
        jc .end

        push si
        call confirm_record_passwd
        pop si
        jc .end
        
        xor byte [si + struc_bootrecord.flags], INFOFLAG_AUTOHIDE
        inc byte [change_occured]               ; some changes occured.

.end:
        ret

;=============================================================================
;mark_active ---- mark the record active
;=============================================================================
mark_active:
        cmp byte [good_record_number], 0
        je .end

        call get_record_pointer
        call check_allow_act
        jc .end

        push si
        call confirm_record_passwd
        pop si
        jc .end

        mov dl, [si + struc_bootrecord.drive_id]

        push si
        lea si, [good_record_list]
        lea di, [boot_records]

        mov cl, [good_record_number]
        xor ch, ch
        mov dh, SIZE_OF_BOOTRECORD
        xor ebx, ebx
.loop_clear_act:                                ; clear all active marks of
        push di                                 ; the boot records in same
        lodsb                                   ; drive and father partition.
        mul dh
        add di, ax
        cmp dl, [di + struc_bootrecord.drive_id]
        jne .do_nothing
        cmp [di + struc_bootrecord.father_abs_addr], ebx
        jne .do_nothing
        and byte [di + struc_bootrecord.flags], ~ INFOFLAG_ACTIVE
.do_nothing:
        pop di
        loop .loop_clear_act

        pop si
        
        call mark_record_active
        jnc .end                                ; mark active ok

        or ax, ax
        jz .cannot_act
        mov si, [str_idx.disk_error]
        jmp .report_error

.cannot_act:
        mov si, [str_idx.mark_act_failed]
.report_error:
        call error_box
.end:
        call draw_menu
        ret

;=============================================================================
;toggle_hidden ---- toggle a record's hidden attribute
;=============================================================================
toggle_hidden:
        cmp byte [good_record_number], 0
        je .end

        call get_record_pointer
        call check_allow_hide
        jc .end

        push si
        call confirm_record_passwd
        pop si
        jc .end

        call toggle_record_hidden
        jnc .end                                 ; toggle hidden ok

        or ax, ax
        jz .cannot_hide
        mov si, [str_idx.disk_error]
        jmp .report_error

.cannot_hide:
        mov si, [str_idx.toggle_hid_failed]
.report_error:
        call error_box
.end:
        ret

;=============================================================================
;delete_record ---- delete a boot record
;=============================================================================
delete_record:
        cmp byte [good_record_number], 0
        je .end

        call confirm_root_passwd
        jc .end_clean

        call get_record_pointer
        test byte [si + struc_bootrecord.flags], INFOFLAG_HIDDEN
        jz .del_it

        call toggle_record_hidden           ; unhide it first.
        jnc .del_it                         ; unhide ok, del it.

        or ax, ax
        jz .cannot_hide
        mov si, [str_idx.disk_error]
        jmp .report_error

.del_it:
        xor al, al
        mov di, si
        mov cx, SIZE_OF_BOOTRECORD
        rep stosb

        inc byte [change_occured]               ; some changes occured.

        call init_good_record_list
        mov al, [good_record_number]
        or al, al
        jz .no_record
        
        dec al
        cmp al, [first_visible_rec]                     ; adjust the cursor
        jae .check_focus_pos                            ; and menu position.
        mov [first_visible_rec], al                     ;
.check_focus_pos:                                       ;
        cmp al, [focus_record]                          ;
        jae .end_clean                                  ;
        mov [focus_record], al                          ;
        jmp .end_clean                                  ;
        
.no_record:
        mov [first_visible_rec], al
        mov [focus_record], al
        jmp .end_clean
        
.cannot_hide:
        mov si, [str_idx.toggle_hid_failed]
.report_error:
        call error_box
        ret
.end_clean:
        call draw_screen
.end:
        ret
        

;=============================================================================
;research_all_drives ---- research all drives for boot records
;=============================================================================
research_all_drives:
        or byte [kernel_flags], KNLFLAG_FLOPPY_OK
        jmp research_records
        
;=============================================================================
;research_fixed_drives ---- research fixed drives for boot records
;=============================================================================
research_fixed_drives:
        and byte [kernel_flags], ~ KNLFLAG_FLOPPY_OK
        
;=============================================================================
;research_records ---- research all drives for boot records
;=============================================================================
research_records:
        call confirm_root_passwd
        jc .end

        mov cl, [good_record_number]
        xor ch, ch
        or cl, cl                               ; if no good record then go to
        jz .init_it                             ; init directly.
        
        lea di, [good_record_list]
        mov dl, SIZE_OF_BOOTRECORD

; unhide all hidden partition first.
.loop_unhide:
        mov al, [di]
        inc di
        mul dl
        lea si, [boot_records]
        add si, ax
        test byte [si + struc_bootrecord.flags], INFOFLAG_HIDDEN
        jz .not_hidden
        call toggle_record_hidden
        jc .hidden_error
.not_hidden:
        loop .loop_unhide
        
.init_it:
        inc byte [change_occured]               ; some changes occured.

        call init_boot_records
        call init_good_record_list

        xor al, al
        mov byte [focus_record], al
        mov byte [first_visible_rec], al
        call draw_screen
        jmp .end
        
.hidden_error:
        or ax, ax
        jz .cannot_hide
        mov si, [str_idx.disk_error]
        jmp .report_error
.cannot_hide:
        mov si, [str_idx.toggle_hid_failed]
.report_error:
        call error_box
.end:
        ret

        
;=============================================================================
;hide_auto_hides ---- hide all partitions that marked auto hide,
;                     except the focus record.
;input:
;      none
;output:
;      cf = 0 success
;      cf = 1 failed
;=============================================================================
hide_auto_hides:
        mov cl, [good_record_number]
        xor ch, ch
        or cl, cl                               ; if no good record then go to
        jz .end_ok                              ; init directly.
        
        lea di, [good_record_list]
        mov dl, SIZE_OF_BOOTRECORD

; hide all auto hide partitions.
.loop_hide:
        mov al, [di]
        inc di

        cmp ch, [focus_record]                  ; do not hide the focus record.
        je .not_hide

        mul dl
        lea si, [boot_records]
        add si, ax
        mov al, [si + struc_bootrecord.flags]
        test al, INFOFLAG_AUTOHIDE
        jz .not_hide
        test al, INFOFLAG_HIDDEN
        jnz .not_hide
        call toggle_record_hidden
        jc .hidden_error
.not_hide:
        inc ch
        cmp ch, cl
        jb .loop_hide
        
.end_ok:
        clc
        ret
        
.hidden_error:
        or ax, ax
        jz .cannot_hide
        mov si, [str_idx.disk_error]
        jmp .report_error
.cannot_hide:
        mov si, [str_idx.toggle_hid_failed]
.report_error:
        call error_box
.end:
        stc
        ret

;=============================================================================
;move_main_win_left ---- move the main window left
;=============================================================================
move_main_win_left:
        mov al, [position.main_win_col]
        or al, al
        jz .no_move
        dec al
        mov [position.main_win_col], al
        call draw_screen
.no_move:
        ret

;=============================================================================
;move_main_win_up ---- move the main window up
;=============================================================================
move_main_win_up:
        mov al, [position.main_win_row]
        cmp al, [size.copyright]
        jbe .no_move
        dec al
        mov [position.main_win_row], al
        call draw_screen
.no_move:
        ret

;=============================================================================
;move_main_win_right ---- move the main window right
;=============================================================================
move_main_win_right:
        mov al, [position.main_win_col]
        mov ah, [screen_width]
        sub ah, MAIN_WIN_WIDTH+2
        cmp al, ah
        jae .no_move
        inc al
        mov [position.main_win_col], al
        call draw_screen
.no_move:
        ret

;=============================================================================
;move_main_win_down ---- move the main window down
;=============================================================================
move_main_win_down:
        mov al, [position.main_win_row]
        mov ah, [screen_height]
;        sub ah, [size.main_win_height]
        sub ah, [size.hint]
        cmp al, ah
        jae .no_move
        inc al
        mov [position.main_win_row], al
        call draw_screen
.no_move:
        ret

;=============================================================================
;set_delay_time ---- set the delay time
;=============================================================================
set_delay_time:
        call confirm_root_passwd
        jc .end

        call draw_screen

        xor ah, ah
        mov al, [color.input_box_msg]
        mov bx, [color.input_box_frame]
        mov cl, 3
        mov si, [str_idx.delay_time]
        lea di, [tmp_buffer]
        mov byte [di], 0
        call input_box
        jc .end_clean

        mov si, di
        mov cl, ch
        xor ch, ch
        call atoi

        cmp ax, 255
        jbe .set_time
        mov al, 255
.set_time:
        mov [delay_time], al
        inc byte [change_occured]               ; some changes occured.
        
.end_clean:
        call draw_screen
.end:
        ret

;=============================================================================
;boot_default ---- boot the default record
;=============================================================================
boot_default:
        mov ah, [default_record]
        lea si, [good_record_list]
        mov cl, [good_record_number]
        xor ch, ch
        or cl, cl
        jz .no_default

.loop_search:
        lodsb
        cmp al, ah
        je .found_it
        loop .loop_search
        
.no_default:                                ; no default record, do nothing.
        ret
        
.found_it:
        call do_boot_record
        ret
        
;=============================================================================
;boot_it ---- boot the selected record
;=============================================================================
boot_it:
        cmp byte [good_record_number], 0
        je .end

        call confirm_record_passwd
        jc .end

        lea si, [good_record_list]
        mov al, [focus_record]
        xor ah, ah
        add si, ax
        lodsb

        call do_boot_record
.end:
        ret

;=============================================================================
;do_boot_record ---- really boot the given record.
;input:
;      al =  the boot record number.
;=============================================================================
do_boot_record:
        mov bl, SIZE_OF_BOOTRECORD
        mul bl

        lea si, [boot_records]
        add si, ax

        push si
        call ask_save_changes                   ; ask if save the changes.
        call hide_auto_hides
        pop si
        jc .end
        
        call boot_the_record
        
        or al, al
        jnz .disk_error

        mov si, [str_idx.no_system]
        jmp .report_error
.disk_error:
        mov si, [str_idx.disk_error]
.report_error:
        call error_box
.end:
        ret
        
;=============================================================================
;reture_to_bios ---- give control back to BIOS
;=============================================================================
return_to_bios:
        call confirm_root_passwd
        jc .end

        call ask_save_changes                   ; ask if save the changes.
        
        call reset_video_mode
        
%ifdef EMULATE_PROG
        mov ax, 0x4c00                          ; exit to dos
        int 0x21                                ;
%else
        int 0x18                                ; return to BIOS
%endif

.end:
        ret

;=============================================================================
; utilitiy functions
;=============================================================================
        
;=============================================================================
;get_record_pointer ---- get current record pointer
;input:
;      none
;output:
;      ds:si -> record pointer
;=============================================================================
get_record_pointer:
        push dx
        push ax
        mov dl, [focus_record]                  ;
        xor dh, dh                              ; get real record index
        lea si, [good_record_list]              ;
        add si, dx                              ;
        lodsb                                   ;

        mov dl, SIZE_OF_BOOTRECORD              ; get the pointer to
        mul dl                                  ; the record.
        lea si, [boot_records]                  ;
        add si, ax                              ;
        pop ax
        pop dx
        ret
        
;=============================================================================
;confirm_root_passwd ---- confirm the root password
;input:
;      none
;output:
;      cf = 0 success
;      cf = 1 failed or cancel
;=============================================================================
confirm_root_passwd:
        mov bx, [root_password]
        mov cx, [root_password+2]
        or bx, bx
        jnz .have_password
        or cx, cx
        jnz .have_password
        jmp .auth_ok
        
.have_password:                                     ; check root
        mov si, [str_idx.root_passwd]               ; password
        call confirm_passwd                         ;
.auth_ok:
        ret


;=============================================================================
;confirm_record_passwd ---- confirm the record password
;=============================================================================
confirm_record_passwd:
        call get_record_pointer
        mov bx, [si + struc_bootrecord.password]
        mov cx, [si + struc_bootrecord.password+2]
        or bx, bx
        jnz .have_password
        or cx, cx
        jnz .have_password
        jmp .auth_ok
        
.have_password:                                     ; check record
        mov si, [str_idx.record_passwd]             ; password
        call confirm_passwd                         ;
.auth_ok:
        ret

;=============================================================================
;confirm_passwd ---- let user input a password and confirm it.
;input:
;      bx:cx = password
;      ds:si -> message string
;output:
;      cf = 0 success
;      cf = 1 failed or cancel
;=============================================================================
confirm_passwd:
        call input_passwd
        jc .cancel

        cmp bx, ax
        jne .cmp_root
        cmp cx, dx
        jne .cmp_root
        jmp .ok
        
.cmp_root:
        cmp [root_password], ax
        jne .failed
        cmp [root_password+2], dx
        jne .failed
.ok:
        clc
        ret
        
.failed:
        mov si, [str_idx.wrong_passwd]
        call error_box
        
.cancel:
        stc
        ret

;=============================================================================
;input_passwd ---- input a passwd
;input:
;      ds:si -> message string
;output:
;      cf = 0 success, ax:dx = password
;      cf = 1 cancel
;=============================================================================
input_passwd:
        push bx
        push cx
        
        mov ah, 1
        mov al, [color.input_box_msg]
        mov bx, [color.input_box_frame]
        mov cl, MAX_PASSWORD_LENGTH
        lea di, [tmp_buffer]
        mov byte [di], 0
        call input_box
        jc .cancel_input

        mov si, di
        mov cl, ch
        xor ch, ch
        call calc_password
        clc
.cancel_input:
        push ax
        push dx
        pushf
        call draw_screen
        popf
        pop dx
        pop ax
        pop cx
        pop bx
        ret

;=============================================================================
;error_box ---- draw error message box.
;input:
;      ds:si -> error message
;output:
;      none
;=============================================================================
error_box:
        call strlen
        mov dl, cl
        add dl, 4
        mov dh, [size.box_height]
        mov al, [color.error_box_msg]
        mov bx, [color.error_box_frame]
        mov di, [str_idx.error]
        xchg di, si
        call message_box
        xor ah, ah
        int 0x16
        push ax
        call draw_screen
        pop ax
        ret

;=============================================================================
;info_box ---- draw infomation message box.
;input:
;      ds:si -> infomation message
;output:
;      none
;=============================================================================
info_box:
        call strlen
        mov dl, cl
        add dl, 4
        mov dh, [size.box_height]
        mov al, [color.info_box_msg]
        mov bx, [color.info_box_frame]
        mov di, [str_idx.info]
        xchg di, si
        call message_box
        xor ah, ah
        int 0x16
        push ax
        call draw_screen
        pop ax
        ret

;=============================================================================
;include area
;=============================================================================

%include "ui.asm"
%include "utils.asm"
%include "knl.asm"
%include "hd_io.asm"

;=============================================================================
; data area
;=============================================================================

func_list:
.hotkeys:
        dw  kbUp
        dw  kbDown
        dw  kbEnter
        dw  kbEsc
        dw  kbF1
        dw  kbCtrlF1
        dw  kbF2
        dw  kbF3
        dw  kbF4
        dw  kbF5
        dw  kbF6
        dw  kbF7
        dw  kbF8
        dw  kbShiftF8
        dw  kbF9
        dw  kbF10
        dw  kbGrayPlus
        dw  kbDel
        dw  kbHome
        dw  kbPgDn
        dw  kbEnd
        dw  kbCtrlD
        dw  kbCtrlI
        dw  kbCtrlH
        dw  kbCtrlT
        dw  kbCtrlQ
        
.entries:
        dw  up_cursor                       ; kbUp
        dw  down_cursor                     ; dbDown
        dw  boot_it                         ; kbEnter
        dw  boot_default                    ; kbEsc
        dw  show_help                       ; kbF1
        dw  show_about                      ; kbCtrlF1
        dw  save_changes                    ; kbF2
        dw  change_name                     ; kbF3
        dw  mark_active                     ; kbF4
        dw  toggle_hidden                   ; kbF5
        dw  toggle_auto_active              ; kbF6
        dw  toggle_auto_hide                ; kbF7
        dw  set_default_record              ; kbF8
        dw  unset_default_record            ; kbShiftF8
        dw  change_record_passwd            ; kbF9
        dw  change_root_passwd              ; kbF10
        dw  change_video_mode               ; kbGrayPlus
        dw  move_main_win_left              ; kbDel
        dw  move_main_win_up                ; kbHome
        dw  move_main_win_right             ; kbPgDn
        dw  move_main_win_down              ; kbEnd
        dw  delete_record                   ; kbCtrlD
        dw  research_all_drives             ; kbCtrlI
        dw  research_fixed_drives           ; kbCtrlH
        dw  set_delay_time                  ; kbCtrlT
        dw  return_to_bios                  ; kbCtrlQ

;END OF KERNEL
        dw BR_GOOD_FLAG
end_of_kernel:

;=============================================================================
;theme data
;=============================================================================
theme_start:

%ifdef THEME_ZH
%include "themes/theme-zh.asm"
%else
%include "themes/theme-us.asm"
%endif

;=============================================================================
; temp data area
;=============================================================================

times MAX_SBM_SIZE - ($-$$) db 0                ; skip enough space for theme.

%ifdef EMULATE_PROG
db 'TEMP DATA',0
%endif

%include "tempdata.asm"
db 'END DATA'

